/*
 * This code was created by Jeff Molofee '99 
 * (ported to Linux/SDL by Ti Leggett '01)
 *
 * If you've found this code useful, please let me know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 * 
 * or for port-specific comments, questions, bugreports etc. 
 * email to leggett@eecs.tulane.edu
 */

/* SDL program to take screenshots: modified by Linh Pham and Joey Turner */
 
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "/home/jturner/include/SDL/SDL.h"
// include the OpenGL program you want to take screenshots of 
#include "report02-leapfrog.cpp"
#include <string.h>

// screen width, height, and bit depth 
#define SCREEN_WIDTH  300
#define SCREEN_HEIGHT 300
#define SCREEN_BPP     16

// Define our booleans 
#define TRUE  1
#define FALSE 0

// This is our SDL surface 
SDL_Surface *surface;

//////////////////////////////////////////////////
// Grab the OpenGL screen and save it as a .tga //
// Copyright (C) Marius Andra 2001              //
// http://cone3d.gz.ee  EMAIL: cone3d@hot.ee    //
//////////////////////////////////////////////////
int screenShot(const char *filename)
{
  // we will store the image data here
  unsigned char *pixels;
  // the thingy we use to write files
  FILE * shot;
  // we get the width/height of the screen into this array
  int screenStats[4];

  // get the width/height of the window
  glGetIntegerv(GL_VIEWPORT, screenStats);

  // generate an array large enough to hold the pixel data 
  // (width*height*bytesPerPixel)
  pixels = new unsigned char[screenStats[2]*screenStats[3]*3];
  // read in the pixel data, TGA's pixels are BGR aligned
  glReadPixels(0, 0, screenStats[2], screenStats[3], GL_BGR, 
                                   GL_UNSIGNED_BYTE, pixels);

  // open the file for writing. If unsucessful, return 1
  if((shot=fopen(filename, "wb"))==NULL) return 1;

  // this is the tga header it must be in the beginning of 
  // every (uncompressed) .tga
  unsigned char TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};
  // the header that is used to get the dimensions of the .tga
  // header[1]*256+header[0] - width
  // header[3]*256+header[2] - height
  // header[4] - bits per pixel
  // header[5] - ?
  unsigned char header[6]={((int)(screenStats[2]%256)),
                   ((int)(screenStats[2]/256)),
                   ((int)(screenStats[3]%256)),
		   ((int)(screenStats[3]/256)),24,0};

  // write out the TGA header
  fwrite(TGAheader, sizeof(unsigned char), 12, shot);
  // write out the header
  fwrite(header, sizeof(unsigned char), 6, shot);
  // write the pixels
  fwrite(pixels, sizeof(unsigned char), 
                 screenStats[2]*screenStats[3]*3, shot);

  // close the file
  fclose(shot);
  // free the memory
  delete [] pixels;

  // return success
  return 0;
}



// function to release/destroy our resources and restoring the old desktop 
void Quit( int returnCode )
{
    // clean up the window 
    SDL_Quit( );

    // and exit appropriately 
    exit( returnCode );
}

// function to reset our viewport after a window resize 
int resizeWindow( int width, int height )
{
    // Height / width ration 
    GLfloat ratio;

    // Protect against a divide by zero 
    if ( height == 0 )
	height = 1;

    ratio = ( GLfloat )width / ( GLfloat )height;

    // Setup our viewport. 
    glViewport( 0, 0, ( GLsizei )width, ( GLsizei )height );

    // change to the projection matrix and set our viewing volume. 
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    // Set our perspective 
    gluPerspective( 45.0f, ratio, 0.1f, 100.0f );

    // Make sure we're chaning the model view and not the projection 
    glMatrixMode( GL_MODELVIEW );

    // Reset The View 
    glLoadIdentity( );

    return( TRUE );
}

// function to handle key press events 
void handleKeyPress( SDL_keysym *keysym )
{
    switch ( keysym->sym )
	{
	case SDLK_ESCAPE:
	    // ESC key was pressed 
	    Quit( 0 );
	    break;
	case SDLK_F1:
	    // F1 key was pressed
	     
	    SDL_WM_ToggleFullScreen( surface );
	    break;
	default:
	    break;
	}

    return;
}

// function that takes an integer and returns a four-element array 
// of chars of that number.  For example: 1 -> 001
char *number(int num)
{
	int numdigits = int(floor(log(num) / log(10)));
	char *str = new char[4];
	str[3] = 0;
	for (int i=2; i>=0; i--)
	{
		if (i > numdigits) str[2-i] = '0';
		else
		{
			int digit = int(num/pow(10,i));
			str[2-i] = (char)(48+digit);
			num -= int(digit*pow(10,i));
		}
	}
	return str;
}
		
		
int main( int argc, char **argv )
{
	int pic = 0;
    // Flags to pass to SDL_SetVideoMode 
    int videoFlags;
    // main loop variable 
    int done = FALSE;
    // used to collect events 
    SDL_Event event;
    // this holds some info about our display 
    const SDL_VideoInfo *videoInfo;
    // whether or not the window is active 
    int isActive = TRUE;

    // initialize SDL 
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    fprintf( stderr, "Video initialization failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    // Fetch the video info 
    videoInfo = SDL_GetVideoInfo( );

    if ( !videoInfo )
	{
	    fprintf( stderr, "Video query failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    // the flags to pass to SDL_SetVideoMode 
    videoFlags  = SDL_OPENGL;          // Enable OpenGL in SDL 
    videoFlags |= SDL_GL_DOUBLEBUFFER; // Enable double buffering 
    videoFlags |= SDL_HWPALETTE;       // Store the palette in hardware 
    videoFlags |= SDL_RESIZABLE;       // Enable window resizing 

    // This checks to see if surfaces can be stored in memory 
    if ( videoInfo->hw_available )
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    // This checks if hardware blits can be done 
    if ( videoInfo->blit_hw )
	videoFlags |= SDL_HWACCEL;

    // Sets up OpenGL double buffering 
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    // get a SDL surface 
    surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
				videoFlags );

    // Verify there is a surface 
    if ( !surface )
	{
	    fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
	    Quit( 1 );
	}

    // resize the initial window 
    resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );
	
	// call myinit() from other program
	myinit();

    // wait for events 
    while ( !done )
	{
	    // handle the events in the queue 

	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			case SDL_ACTIVEEVENT:
			    // Something's happend with our focus
			     
			    if ( event.active.gain == 0 )
				isActive = FALSE;
			    else
				isActive = TRUE;
			    break;			    
			case SDL_VIDEORESIZE:
			    // handle resize event 
			    surface = SDL_SetVideoMode( event.resize.w,
							event.resize.h,
							16, videoFlags );
			    if ( !surface )
				{
				    fprintf( stderr, "Could not get a surface after resize: %s\n", SDL_GetError( ) );
				    Quit( 1 );
				}
			    resizeWindow( event.resize.w, event.resize.h );
			    break;
			case SDL_KEYDOWN:
			    // handle key presses 
			    handleKeyPress( &event.key.keysym );
			    break;
			case SDL_QUIT:
			    // handle quit requests 
			    done = TRUE;
			    break;
			default:
			    break;
			}
		}

	    // draw the scene 
	    if ( isActive )

		// call the display() function from the other program
		display( );
		char *filename = new char[13];
		strcat(filename, "/img/");
		strcat(filename, number(pic++));
		strcat(filename, ".tga");
		const char *temp = filename;

		// call the screenShot function from above and save it into file called filename
		screenShot(temp);

		// don't save more than one hundred pictures
		if(pic > 100) Quit( 0 );
	}

    // clean ourselves up and exit 
    Quit( 0 );

    // Should never get here 
    return( 0 );
}
